from lxml import etree

SPARQLNS = u'http://www.w3.org/2005/sparql-results#'

def parse_sparql_result(xml):
    doc = etree.fromstring(xml)
    results = []
    for result in doc.xpath('/s:sparql/s:results/s:result',
                            namespaces={'s': SPARQLNS}):
        data = {}
        for binding in result:
            name = unicode(binding.attrib['name'])
            for value in binding:
                type = unicode(value.tag.split('}')[-1])
                lang = value.attrib.get(
                    '{http://www.w3.org/XML/1998/namespace}lang')
                datatype = value.attrib.get('datatype')
                text = value.text
                if not text is None and not isinstance(text, unicode):
                    text = text.decode('utf8')

                if type == 'uri':
                    # allegro graph returns context uri's with <> chars
                    if text.startswith('<'):
                        text = text[1:]
                    if text.endswith('>'):
                        text = text[:-1]
                    
                data[name] = {'value': text,
                              'type': type}
                if not lang is None:
                    data[name]['xml:lang'] = lang.decode('utf8')
                if not datatype is None:
                    data[name]['datatype'] = datatype.decode('utf8')

        results.append(data)
    return results



from zope.interface import Interface, Attribute

class IConnector(Interface):
    def connect():
        """
        Connect to database, retrieve an IDatabase instance
        """
    def disconnect(db):
        """
        Disconnect the database
        """

class IDatabase(Interface):

    def formats():
        """
        Return a list of string identifiers describing the
        different rdf serialization formats that the database
        supports. Examples are:
        ntriples, rdfxml, rdfxml-abbrev, turtle, n3
        """
        
    def contexts():
        """
        Return a list of context names
        """
    
    def add(file, format_name, base_uri, context_name):
        """
        Add triples from filestream with triples data in a
        specific format. A base_uri should be supplied.
        Triples are always added in a specific context
        """

    def remove(file, format_name, base_uri, context_name):
        """
        Remove triples from a filestream with triples data in
        a specifc format.
        """

    def serialize(format_name, context_name):
        """
        Return a filestream object with triples data in a
        specific format
        """
        
    def clear(context_name):
        """
        Remove all triples from a context
        """

    def register_prefix(prefix, namespace):
        """
        Register a namespace with a specific prefix
        These will be used in serializing rdfxml
        if backend supports this
        """

    def count(context_name=None):
        """
        Return the number of triples in the database or
        None if the backend does not support this
        Optionally a context_name can be specified
        """

    def select(sparql_query):
        """
        Run a sparql SELECT query, returns a list
        of dictionaries in sparql result format (json-like)
        """

    def ask(sparql_query):
        """
        Run a sparql ASK query, returns a boolean
        """
    def construct(sparql_query, format):
        """
        Run a sparql CONSTRUCT query, returns a
        filestream with triples in the specifed format
        or None if no result was found
        """
        




from sparrow.redland_backend import RedlandConnector
from sparrow.rdflib_backend import RDFLibConnector
from sparrow.sesame_backend import SesameConnector
from sparrow.allegro_backend import AllegroConnector

def database(backend, dburi):
    if backend == 'redland':
        return RedlandConnector(dburi)
    elif backend == 'rdflib':
        return RDFLibConnector(dburi)
    elif backend == 'sesame':
        return SesameConnector(dburi)
    elif backend == 'allegro':
        return AllegroConnector(dburi)        
    else:
        raise ValueError('Unknown database backend: "%s"' % backend)



import os
from os.path import join
import shutil
import subprocess
from cStringIO import StringIO
import socket
from urllib import quote, urlencode

import httplib2
from zope.interface import implements
from lxml import etree

from sparrow.error import ConnectionError, DatabaseError, QueryError
from sparrow.interfaces import IConnector, IDatabase
from sparrow.utils import parse_sparql_result


class SesameConnector(object):
    implements(IConnector)
        
    def __init__(self, dburi):
        self._host, rest = dburi[7:].split(':', 1)
        self._port, self._name = rest.split('/', 1)
        self._url = 'http://%s:%s/openrdf-sesame' % (self._host, self._port)
        http = httplib2.Http()
        try:
            resp, content = http.request(
                '%s/repositories' % self._url,
                "GET",
                headers={'Accept': 'application/sparql-results+xml'})
        except socket.error:
            raise ConnectionError(
                'Can not connect to repository, is it running?')
        
        if resp['status'] != '200':
            raise ConnectionError(
                'Can not connect to server: %s' % resp['status'])
            
        for repo in parse_sparql_result(content):
            if repo['id']['value'] == self._name:
                break
        else:
            raise ConnectionError(
                'Server has no repository: %s' % self._name)
        
    def connect(self):
        return SesameDatabase(self._url, self._name)
    
    def disconnect(self, db):
        del db._http

class SesameDatabase(object):
    implements(IDatabase)
    
    def __init__(self, sesame_url, db_name):
        self._url = sesame_url
        self._name = db_name
        self._http = httplib2.Http()
        
    def formats(self):
        return ['ntriples', 'rdfxml', 'turtle', 'n3']
    
    def contexts(self):
        resp, content = self._http.request(
            '%s/repositories/%s/contexts' % (self._url, self._name),
            "GET",
            headers={'Accept': 'application/sparql-results+xml'})

        return [c['contextID']['value'].split(':', 1)[1] for c in
                parse_sparql_result(content)]

    def _get_mimetype(self, format):
        return {'ntriples': 'text/plain',
                'rdfxml': 'application/rdf+xml',
                'turtle': 'application/x-turtle',
                'n3': 'text/rdf+n3',
                'trix': 'application/trix',
                'trig': 'applcation/x-trig'}[format]

    def _get_context(self, context):
        return '<context:%s>' % context
        
    def register_prefix(self, prefix, namespace):        
        clength = str(len(namespace))
        resp, content = self._http.request(
            '%s/repositories/%s/namespaces/%s' % (
                self._url, self._name, prefix),
            'PUT',
            body=namespace,
            headers = {"Content-length": clength})
        if resp['status'] != '204':
            raise DatabaseError(content)

    def add(self, file, format, base_uri, context):
        data = file.read()
        clength = str(len(data))
        ctype = self._get_mimetype(format)
        params = urlencode({'context': self._get_context(context),
                            'baseURI': '<%s>' % base_uri})
        resp, content = self._http.request(
            '%s/repositories/%s/statements?%s' % (
                self._url, self._name, params),
            'POST',
            body=data,
            headers = {"Content-type": ctype,
                       "Content-length": clength})
        
        if resp['status'] != '204':
            raise DatabaseError(content)       
        
    def serialize(self, format, context):
        context = quote(self._get_context(context))
        ctype = self._get_mimetype(format)
        
        resp, content = self._http.request(
            '%s/repositories/%s/statements?context=%s' % (
                self._url, self._name, context),
            'GET',
            headers = {"Accept": ctype})
        if resp['status'] != '200':
            raise DatabaseError(content)

        return StringIO(content)
        
    def remove(self, file, format, base_uri, context):
        data = file.read()
        clength = str(len(data))
        ctype = self._get_mimetype(format)
        params = urlencode({'context': self._get_context(context),
                            'baseURI': '<%s>' % base_uri})
        resp, content = self._http.request(
            '%s/repositories/%s/statements?%s' % (
                self._url, self._name, params),
            'DELETE',
            body=data,
            headers = {"Content-type": ctype,
                       "Content-length": clength})
        if resp['status'] != '204':
            raise DatabaseError(content)
    
    def clear(self, context):
        context = quote(self._get_context(context))
        resp, content = self._http.request(
            '%s/repositories/%s/statements?context=%s' % (
                self._url, self._name, context),
            'DELETE')
        if resp['status'] != '204':
            raise DatabaseError(content)
        
    def count(self, context=None):
        if context is None:
            context = ''
        else:
            context = '?context='+quote(self._get_context(context))
        resp, content = self._http.request(
            '%s/repositories/%s/size%s' % (
                self._url, self._name, context),
            'GET')
        if resp['status'] != '200':
            raise DatabaseError(content)
        return int(content)
    
    def select(self, sparql):
        params = urlencode({'query': sparql,
                            'queryLn': 'SPARQL',
                            'infer':'false'})
        
        resp, content = self._http.request(
            '%s/repositories/%s?%s' % (
            self._url, self._name, params),
            'GET',
            headers={'Accept': 'application/sparql-results+xml'})
        
        if resp['status'] != '200':
            raise QueryError(content)

        return parse_sparql_result(content)
    
    def ask(self, sparql):
        params = urlencode({'query': sparql,
                            'queryLn': 'SPARQL',
                            'infer':'false'})
        
        resp, content = self._http.request(
            '%s/repositories/%s?%s' % (
            self._url, self._name, params),
            'GET',
            headers={'Accept': 'text/boolean'})

        if resp['status'] != '200':
            raise QueryError(content)

        if content == 'true':
            return True
        elif content == 'false':
            return False
        else:
            raise QueryError('ASK query got unexpected result: %s' % content)
    
    def construct(self, sparql, format):
        ctype = self._get_mimetype(format)        
        params = urlencode({'query': sparql,
                            'queryLn': 'SPARQL',
                            'infer':'false'})
        
        resp, content = self._http.request(
            '%s/repositories/%s?%s' % (
            self._url, self._name, params),
            'GET',
            headers={'Accept': ctype})
        
        if resp['status'] != '200':
            raise QueryError(content)

        return StringIO(content)

def start_server(host, port, uri, id, title, path):
    tomcat_dir = join(path ,'parts', 'tomcat-install')

    # check if port and host are configured in tomcats server.xml
    server_conf = join(tomcat_dir, 'conf', 'server.xml')
    doc = etree.parse(server_conf)
    connector_el = doc.xpath('//Service[@name="Catalina"]/Connector')[0]
    if int(connector_el.attrib['port']) != port:
        connector_el.attrib['port'] = str(port)
        doc.write(server_conf)
    engine_el = doc.xpath(
        '//Service[@name="Catalina"]/Engine[@name="Catalina"]')[0]
    if engine_el.attrib['defaultHost'] != host:
        engine_el.attrib['defaultHost'] = host

        host_el = engine_el.xpath('Host')[0]
        if host_el.attrib['name'] != host:
            host_el.attrib['name'] = host
        doc.write(server_conf)
    
    # copy the war files to tomcat
    sesame_dir = join(path , 'parts', 'sesame-install')
    if not os.path.isfile(join(tomcat_dir, 'webapps', 'openrdf-sesame.war')):
        shutil.copyfile(join(sesame_dir, 'war', 'openrdf-sesame.war'),
                        join(tomcat_dir, 'webapps', 'openrdf-sesame.war'))
    if not os.path.isfile(join(tomcat_dir, 'webapps', 'openrdf-workbench.war')):
        shutil.copyfile(join(sesame_dir, 'war', 'openrdf-workbench.war'),
                        join(tomcat_dir, 'webapps', 'openrdf-workbench.war'))
    
    
    catalina = join(tomcat_dir ,'bin', 'catalina.sh')
    
    os.system('%s run' % catalina)


SESAME_TEST_TEMPLATE = """
drop test.
yes
create memory.
test
Test Repository
false
0
"""

SESAME_MEMORY_TEMPLATE = """
create memory.
%(id)s
%(title)s
false
0
"""

SESAME_NATIVE_TEMPLATE = """
create native.
%(id)s
%(title)s
spoc, posc, opsc
"""

SESAME_MYSQL_TEMPLATE = """
create mysql.
%(id)s
%(title)s
com.mysql.jdbc.Driver
%(host)s
%(port)s
%(db)s
0
%(user)s
%(pwd)s
256
"""

def configure_server(host, port, uri, id, title, path):

    variables = {'id': id, 'title':title}
    
    if uri == 'memory':
        template = SESAME_MEMORY_TEMPLATE % variables
    elif uri == 'native':
        template = SESAME_NATIVE_TEMPLATE % variables
    elif uri.startswith('mysql://'):
        uri = uri[8:]
        userpart, dbpart = uri.split('@')
        user, pwd = userpart.split(':')
        host, db = dbpart.split('/', 1)
        if ':' in host:
            host, port = host.split(':', 1)
        else:
            port = '3306'
        variables['user'] = user
        variables['pwd'] = pwd
        variables['db'] = db
        variables['host'] = host
        variables['port'] = port
        template = SESAME_MYSQL_TEMPLATE % variables
    else:
        raise ValueError('Unknown Sesame backend URI: %s' % uri)
        
    conf_script = join(path, 'parts', 'sesame-install', 'bin', 'console.sh')
    proc = subprocess.Popen(conf_script, shell=True, stdin=subprocess.PIPE)
    # connect to sesame
    proc.communicate("connect http://%s:%s/openrdf-sesame." % (host, port)+
                     SESAME_TEST_TEMPLATE +
                     template)



from unittest import TestCase, TestSuite, makeSuite, main

import sparrow
from sparrow.error import ConnectionError
from sparrow.tests.base_tests import (DatabaseConnectionTest,
                                      DatabaseTest,
                                      DatabaseQueryTest,
                                      open_test_file)

class RDFLibConnectionTest(DatabaseConnectionTest):
    def setUp(self):
        self.connector = sparrow.database('rdflib', 'memory')

    def tearDown(self):
        del self.connector

class RDFLibTest(DatabaseTest):
    def setUp(self):
        self.connector = sparrow.database('rdflib', 'memory')
        self.db = self.connector.connect()

    def tearDown(self):
        self.connector.disconnect(self.db)
        del self.db
        del self.connector

class RDFLibQueryTest(DatabaseQueryTest):
    def setUp(self):
        self.connector = sparrow.database('rdflib', 'memory')
        self.db = self.connector.connect()
        fp = open_test_file('ntriples')
        self.db.add(fp, 'ntriples','file://wine.nt', 'test')
        fp.close()
        
    def tearDown(self):
        self.connector.disconnect(self.db)
        del self.db
        del self.connector
    

def test_suite():
    try:
        sparrow.database('rdflib', 'memory')
    except ConnectionError:
        # rdflib not installed?
        return TestSuite()
    suite = TestSuite()
    suite.addTest(makeSuite(RDFLibConnectionTest))
    suite.addTest(makeSuite(RDFLibTest))
    suite.addTest(makeSuite(RDFLibQueryTest))
    return suite

if __name__ == '__main__':
    main(defaultTest='test_suite')



import os
import StringIO
from unittest import TestCase, TestSuite, makeSuite, main

from sparrow.interfaces import IConnector, IDatabase
from sparrow.error import DatabaseError, QueryError

TESTFILE = 'wine'

def open_test_file(format):
    extension = {'rdfxml': '.rdf',
                 'ntriples': '.nt',
                 'turtle': '.ttl'}
    filename = 'wine' + extension[format]
    return open(
        os.path.join(os.path.dirname(__file__), filename), 'r')

class DatabaseConnectionTest(TestCase):
    def test_connect(self):
        self.assertTrue(IConnector.providedBy(self.connector))
        db = self.connector.connect()
        self.assertTrue(IDatabase.providedBy(db))
        self.connector.disconnect(db)

class DatabaseTest(TestCase):

    def test_broken_parsing(self):
        context = 'test'
        base_uri = 'http://example.org'
        for format in self.db.formats():
            # we use '@' as fake data, because 'foo'
            # does not raise any errors in rdflib
            self.assertRaises(
                DatabaseError,
                self.db.add,
                StringIO.StringIO('@'),
                format, base_uri, context)

    def test_rdfxml_parsing(self):
        formats = self.db.formats()
        if 'rdfxml' in formats:
            self._parse('rdfxml')

    def test_ntriples_parsing(self):
        formats = self.db.formats()
        if 'ntriples' in formats:
            self._parse('ntriples')

    def test_turtle_parsing(self):
        formats = self.db.formats()
        if 'turtle' in formats:
            self._parse('turtle')

    def test_base_uri_parsing(self):
        if not 'rdfxml' in self.db.formats():
            return
        data = StringIO.StringIO('''<?xml version="1.0"?>
<rdf:RDF xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#">
 <rdf:Description rdf:about="">
  <foo:name xmlns:foo="http://foobar.com#">bar</foo:name>
 </rdf:Description>
</rdf:RDF>''')
        self.db.add(data, 'rdfxml', 'http://example.org', 'test')
        result = self.db.serialize('ntriples', 'test').read()
        self.assertEquals(result.strip()[:-1].strip(),
                          '<http://example.org> <http://foobar.com#name> "bar"')
        
    def test_remove_statements(self):
        self._parse()

        fp = StringIO.StringIO(
            '<http://www.w3.org/TR/2003/PR-owl-guide-20031209/wine> '
            '<http://www.w3.org/2000/01/rdf-schema#label> '
            '"Wine Ontology" .\n')
        
        self.db.remove(fp, 'ntriples', 'file://wine.rdf', 'test')
        fp = self._serialize('ntriples')
        data = fp.read()
        self.assertTrue('Wine Ontology' not in data)

    def test_ntriples_serializing(self):
        formats = self.db.formats()
        if not 'ntriples' in formats:
            return

        self._parse()
        fp = self._serialize('ntriples')
        data = fp.read()
        self.assertTrue('Wine Ontology' in data)

    def test_rdfxml_serializing(self):
        formats = self.db.formats()
        if not 'rdfxml' in formats:
            return

        self._parse()
        self.db.register_prefix(
            'vin',
            'http://www.w3.org/TR/2003/PR-owl-guide-20031209/wine#')
        fp = self._serialize('rdfxml')
        data = fp.read()
        fp.close()
        self.assertTrue('Wine Ontology' in data)
        self.assertTrue('xmlns:vin' in data)

    def test_turtle_serializing(self):
        formats = self.db.formats()
        if not 'turtle' in formats:
            return
        
        self._parse()
        self.db.register_prefix(
            'vin',
            'http://www.w3.org/TR/2003/PR-owl-guide-20031209/wine#')
        fp = self._serialize('turtle')
        data = fp.read()
        fp.close()
        self.assertTrue('Wine Ontology' in data)
        self.assertTrue('@prefix vin' in data)
        
    def test_clear(self):
        count = self.db.count('test')
        if count is None:
            return
        self.assertEquals(count , 0)
        self._parse()
        count = self.db.count()
        self.assertTrue(count > 1500)
        self.db.clear('test')
        count = self.db.count()
        self.assertEquals(count , 0)

    def test_contexts(self):
        self.assertEquals(list(self.db.contexts()), [])
        self._parse(format=None, context='a')
        self._parse(format=None, context='b')
        self._parse(format=None, context='c')
        self.assertEquals(sorted(list(self.db.contexts())), [u'a', u'b', u'c'])
        self.db.clear('b')
        self.assertEquals(sorted(list(self.db.contexts())), [u'a', u'c'])
        self.db.clear('a')
        self.db.clear('c')
        self.assertEquals(list(self.db.contexts()), [])

    def _serialize(self, format=None, context='test'):
        if format is None:
            formats = self.db.formats()
            for format in formats:
                if format in ['ntriples', 'rdfxml', 'turtle']:
                    break
                else:
                    return
        return self.db.serialize(format, context)
    
    def _parse(self, format=None, context='test'):
        if format is None:
            formats = self.db.formats()
            for format in formats:
                if format in ['ntriples', 'rdfxml', 'turtle']:
                    break
                else:
                    return
        fp = open_test_file(format)
        base_uri = 'file://wine.rdf'
        self.db.add(fp, format, base_uri, context)
        fp.close()
        
        triples = self.db.count(context) or self.db.count()
        if not triples is None:
            self.assertTrue(triples > 1500)
        
    
class DatabaseQueryTest(TestCase):
    def test_ask(self):
        q = """
        PREFIX rdfs: <http://www.w3.org/2000/01/rdf-schema#>        
        ASK { ?x rdfs:label "Wine Ontology"}
        """
        self.assertTrue(self.db.ask(q))
        q = """
        PREFIX rdfs: <http://www.w3.org/2000/01/rdf-schema#>        
        ASK { ?x rdfs:label "FooBar"}
        """
        self.assertFalse(self.db.ask(q))

    def test_select(self):
        q = """
        PREFIX rdfs: <http://www.w3.org/2000/01/rdf-schema#>
        PREFIX vin: <http://www.w3.org/TR/2003/PR-owl-guide-20031209/wine#>
        SELECT ?grape
        WHERE { ?grape a vin:WineGrape .}
        """
        result = self.db.select(q)
        self.assertEquals(len(result), 16)
        grapes = sorted(
            [r['grape']['value'].split('#')[-1] for r in result])
        
        self.assertEquals(grapes, [
            u'CabernetFrancGrape', u'CabernetSauvignonGrape',
            u'ChardonnayGrape', u'CheninBlancGrape', u'GamayGrape',
            u'MalbecGrape', u'MerlotGrape', u'PetiteSyrahGrape',
            u'PetiteVerdotGrape', u'PinotBlancGrape', u'PinotNoirGrape',
            u'RieslingGrape', u'SangioveseGrape', u'SauvignonBlancGrape',
            u'SemillonGrape', u'ZinfandelGrape'])

    def test_select_literal_language(self):
        q = """
        PREFIX rdfs: <http://www.w3.org/2000/01/rdf-schema#>
        PREFIX vin: <http://www.w3.org/TR/2003/PR-owl-guide-20031209/wine#>
        SELECT ?label
        WHERE { vin:Wine rdfs:label ?label .}
        """
        result = self.db.select(q)
        self.assertEquals(sorted(result),
                          [{u'label': {'type': u'literal',
                                       'value': u'vin',
                                       'xml:lang': u'fr'}},
                           {u'label': {'type': u'literal',
                                       'value': u'wine',
                                       'xml:lang': u'en'}}])

    def test_select_literal_datatype(self):
        q = """
        PREFIX rdfs: <http://www.w3.org/2000/01/rdf-schema#>
        PREFIX vin: <http://www.w3.org/TR/2003/PR-owl-guide-20031209/wine#>
        SELECT ?year
        WHERE { vin:Year1998 vin:yearValue ?year .}
        """
        result = self.db.select(q)
        self.assertEquals(
            sorted(result),
            [{u'year': {
            'type': u'literal',
            'value': u'1998',
            'datatype': u'http://www.w3.org/2001/XMLSchema#positiveInteger'}}])
        
    def test_construct(self):
        q = """
        PREFIX rdfs: <http://www.w3.org/2000/01/rdf-schema#>        
        CONSTRUCT {?x rdfs:label "Wine Ontology"}
        WHERE { ?x rdfs:label "Wine Ontology" .}
        """
        fp = self.db.construct(q, 'ntriples')
        self.assertEquals(
            fp.read().strip()[:-1].strip(),
            ('<http://www.w3.org/TR/2003/PR-owl-guide-20031209/wine> '
             '<http://www.w3.org/2000/01/rdf-schema#label> "Wine Ontology"'))
        fp.close()
        
    def test_broken_query(self):
        self.assertRaises(QueryError,
                          self.db.select,
                          'foo')
        self.assertRaises(QueryError,
                          self.db.ask,
                          'foo')
        self.assertRaises(QueryError,
                          self.db.construct,
                          'foo', 'rdfxml')
        



import os, sys
from os.path import join
from urllib import quote, urlencode

import httplib2
from zope.interface import implements

from sparrow.error import ConnectionError, DatabaseError, QueryError
from sparrow.interfaces import IConnector, IDatabase
from sparrow.utils import parse_sparql_result
from sparrow.sesame_backend import SesameDatabase

class AllegroConnector(object):
    implements(IConnector)
        
    def __init__(self, dburi):
        self._host, rest = dburi[7:].split(':', 1)
        self._port, self._name = rest.split('/', 1)
        
        self._url = 'http://%s:%s/sesame' % (self._host, self._port)
        http = httplib2.Http()
        params = urlencode({'id': self._name,
                            'if-exists': 'open'})
        try:
            resp, content = http.request(
                '%s/repositories?%s' % (self._url, params),
                "POST")
        except socket.error:
            raise ConnectionError(
                'Can not connect to repository, is it running?')
        
        if resp['status'] != '204':
            raise ConnectionError(
                'Can not connect to server: %s' % resp['status'])
        
    def connect(self):
        return AllegroDatabase(self._url, self._name)
    
    def disconnect(self, db):
        del db._http

class AllegroDatabase(SesameDatabase):
    def formats(self):
        return ['rdfxml', 'ntriples']


def start_server(host, port, path):
    part_dir = join(path ,'parts', 'allegro-install')
    allegro_dir = join(part_dir,
                       [d for d in os.listdir(part_dir) if
                        d.startswith('agraph')][0])

    allegro_cmd = join(allegro_dir, 'AllegroGraphServer')
    os.system('%s --http-port %s' % (allegro_cmd, port))
    fp = open(join(allegro_dir, 'agraph.pid'), 'r')
    pid = fp.read().strip()
    fp.close()
    
    print >> sys.stderr, 'Started Allegro server on port %s (pid %s)' % (
        port, pid)
    
    
